@@ -896,6 +896,58 @@ void stream_receiver_check_all_nodes_from_poll(struct stream_thread *sth, usec_t
         if (m->type != POLLFD_TYPE_RECEIVER) continue;
         struct receiver_state *rpt = m->rpt;
 
+        // Probe socket to detect dead connections (e.g., from TCP keepalive)
+        // Uses nd_sock_peek_nowait() which handles both SSL and plain TCP:
+        // - For SSL: uses SSL_peek() to avoid corrupting SSL state
+        // - For plain TCP: uses recv(MSG_PEEK | MSG_DONTWAIT)
+        char probe_byte;
+        ssize_t probe_rc = nd_sock_peek_nowait(&rpt->sock, &probe_byte, 1);
+        if (probe_rc == 0 || (probe_rc < 0 && errno == ECONNRESET)) {
+            // Connection closed by remote (gracefully or via reset)
+            ND_LOG_STACK lgs[] = {
+                ND_LOG_FIELD_TXT(NDF_SRC_IP, rpt->remote_ip),
+                ND_LOG_FIELD_TXT(NDF_SRC_PORT, rpt->remote_port),
+                ND_LOG_FIELD_TXT(NDF_NIDL_NODE, rpt->hostname),
+                ND_LOG_FIELD_CB(NDF_SRC_TRANSPORT, stream_receiver_log_transport, rpt),
+                ND_LOG_FIELD_CB(NDF_SRC_CAPABILITIES, stream_receiver_log_capabilities, rpt),
+                ND_LOG_FIELD_END(),
+            };
+            ND_LOG_STACK_PUSH(lgs);
+
+            worker_is_busy(WORKER_STREAM_JOB_DISCONNECT_REMOTE_CLOSED);
+            nd_log(NDLS_DAEMON, NDLP_ERR,
+                   "STREAM RCV[%zu] '%s' [from %s]: socket closed by remote - closing connection",
+                   sth->id, rrdhost_hostname(rpt->host), rpt->remote_ip);
+
+            stream_receiver_remove(sth, rpt, STREAM_HANDSHAKE_DISCONNECT_SOCKET_CLOSED_BY_REMOTE);
+            continue;
+        }
+        if (probe_rc < 0 && errno != EAGAIN && errno != EWOULDBLOCK) {
+            // Socket error detected (keepalive timeout, etc.)
+            // Save errno immediately as subsequent calls may modify it
+            int saved_errno = errno;
+
+            ND_LOG_STACK lgs[] = {
+                ND_LOG_FIELD_TXT(NDF_SRC_IP, rpt->remote_ip),
+                ND_LOG_FIELD_TXT(NDF_SRC_PORT, rpt->remote_port),
+                ND_LOG_FIELD_TXT(NDF_NIDL_NODE, rpt->hostname),
+                ND_LOG_FIELD_CB(NDF_SRC_TRANSPORT, stream_receiver_log_transport, rpt),
+                ND_LOG_FIELD_CB(NDF_SRC_CAPABILITIES, stream_receiver_log_capabilities, rpt),
+                ND_LOG_FIELD_END(),
+            };
+            ND_LOG_STACK_PUSH(lgs);
+
+            worker_is_busy(WORKER_STREAM_JOB_DISCONNECT_SOCKET_ERROR);
+            nd_log(NDLS_DAEMON, NDLP_ERR,
+                   "STREAM RCV[%zu] '%s' [from %s]: socket error detected: %s - closing connection",
+                   sth->id, rrdhost_hostname(rpt->host), rpt->remote_ip, strerror(saved_errno));
+
+            stream_receiver_remove(sth, rpt, STREAM_HANDSHAKE_DISCONNECT_SOCKET_ERROR);
+            continue;
+        }
+        // probe_rc > 0: data available (normal)
+        // probe_rc < 0 with EAGAIN/EWOULDBLOCK: no data but connection alive
+
         spinlock_lock(&rpt->thread.send_to_child.spinlock);
         STREAM_CIRCULAR_BUFFER_STATS stats = *stream_circular_buffer_stats_unsafe(rpt->thread.send_to_child.scb);
         spinlock_unlock(&rpt->thread.send_to_child.spinlock);
@@ -1073,6 +1125,7 @@ static void stream_receiver_replication_reset(RRDHOST *host) {
 
     __atomic_store_n(&host->stream.rcv.status.replication.counter_in, 0, __ATOMIC_RELAXED);
     __atomic_store_n(&host->stream.rcv.status.replication.counter_out, 0, __ATOMIC_RELAXED);
+    __atomic_store_n(&host->stream.rcv.status.replication.backfill_pending, 0, __ATOMIC_RELAXED);
 }
 
 bool rrdhost_set_receiver(RRDHOST *host, struct receiver_state *rpt) {
